# -*- coding: utf-8 -*-
"""
tz_rendering
~~~~~~~~

HTML helper to render timezones. The output will be a SELECT element.

Will auto guess the user's timezone based on IP. Will auto-select the current selected timezone.

Example usage (returns HTML based on current properties)::

    html_timezones = tz_rendering.html_render_timezones('timezone',
                                                        current_selected,
                                                        get_current_ip(),
                                                        first_entry=_('Select your timezone'))

:copyright: 2012 by Amir Salihefendic ( http://amix.dk/ )
:license: MIT
"""
import json

from . import tz_utils, zones


def html_render_timezones(
    select_name,
    current_selected=None,
    user_ip=None,
    first_entry="Select your timezone",
    force_current_selected=False,
    select_id=None,
    default_timezone=None,
):
    """Render timezone and output HTML.

    `select_name`:
        Is the name of the select element, e.g. <select name="%(select_name)s">.

    `current_selected` (optional):
        Is the name of the current timezone, e.g. "Europe/Copenhagen"
        is used to auto select the current timezone option in the select element

    `user_ip` (optional):
        User's ip, used to auto guess the timezone

    `first_entry` (optional):
        What should the first option be? If `None` it won't be shown at all

    `force_current_selected` (optional):
        Force a show in the top of user's current timezone

    `select_id`:
        Select's elements id, e.g. <select id="%(select_id)s">.
    """

    # Makes it possible to only mark one timezone as selected
    sel_checker = {"non_selected_yet": True}

    def render_option(value, name, selected=False):
        if selected and sel_checker["non_selected_yet"]:
            is_selected = 'selected="selected"'
            sel_checker["non_selected_yet"] = False
        else:
            is_selected = ""
        return '<option value="%s" %s>%s</option>' % (value, is_selected, name)

    def render_option_disabled():
        return '<option disabled="disabled">--------------------</option>'

    if select_id:
        select_elm = '<select name="%s" id="%s">' % (select_name, select_id)
    else:
        select_elm = '<select name="%s">' % select_name

    result = [select_elm]

    if first_entry:
        result.append('<option value="">%s</option>' % first_entry)
        result.append(render_option_disabled())

    if force_current_selected and current_selected:
        timezone = format_tz(current_selected)
        if timezone:
            result.append(render_option(timezone[1], timezone[2], True))
            result.append(render_option_disabled())

    # Guess user's timezone by user_ip
    if user_ip:
        gussed_timezone = tz_utils.guess_timezone_by_ip(user_ip)

        if not gussed_timezone and default_timezone:
            gussed_timezone = format_tz(default_timezone)

        if gussed_timezone:
            is_set = current_selected == gussed_timezone[1] or current_selected == None
            result.append(render_option(gussed_timezone[1], gussed_timezone[2], is_set))
            result.append(render_option_disabled())

    for tz in zones.get_timezones(only_us=True):
        result.append(render_option(tz[1], tz[2], current_selected == tz[1]))

    result.append(render_option_disabled())

    for tz in zones.get_timezones():
        result.append(render_option(tz[1], tz[2], current_selected == tz[1]))

    result.append(render_option_disabled())

    for tz in zones.get_timezones(only_fixed=True):
        result.append(render_option(tz[1], tz[2], current_selected == tz[1]))

    result.append("</select>")

    return "\n".join(result)


def get_timezones_json():
    result = []
    for tz in zones.get_timezones(only_us=True):
        result.append((tz[1], tz[2]))

    for tz in zones.get_timezones():
        result.append((tz[1], tz[2]))

    for tz in zones.get_timezones(only_fixed=True):
        result.append((tz[1], tz[2]))

    return json.dumps(result)


def format_tz(tz_name):
    tz = zones.get_timezones_dict().get(tz_name)
    if tz:
        return tz
    return tz_utils.format_tz_by_name(tz_name)



"""
zones
~~~~~~~~

Holds a collection of common timezones.
Is much smaller and better formated than pytz.common_timezones.
It also supports fixed timezones such as `GMT +7:00`.

Example usage (returns US based timezones)::

    for tz_offset, tz_name, tz_formated in zones.get_timezones(only_us=True):
        print(tz_formated)

        =>

        "(GMT-1000) Hawaii')"
        "(GMT-0900) Alaska')"
        "(GMT-0800) Pacific Time (US & Canada)"
        ...

:copyright: 2012 by Amir Salihefendic ( http://amix.dk/ )
:license: MIT
"""
import re

from timezones import tz_utils, _defs

_UPDATED_TZS = False

_RE_TZ_OFFSET = re.compile(r"([+-])(\d\d)(\d\d)")


def get_timezones(only_us=False, only_fixed=False):
    """Returns an iterator of timezones.

    `only_us` (optional, defaults to `False`):
        Only return US related timezones

    `only_fixed` (optional, defaults to `False`):
        Only return fixed timezones
    """
    global _UPDATED_TZS

    # We need to update the offsets to ensure they are correct
    # with pytz latest info
    if not _UPDATED_TZS:
        _US_TIMEZONES = _update_offsets(_defs._US_TIMEZONES)
        _ALL_TIMEZONES = _update_offsets(_defs._ALL_TIMEZONES)
        _UPDATED_TZS = True

    if only_us:
        for tz in _defs._US_TIMEZONES:
            yield tz
    elif only_fixed:
        for tz in _defs._FIXED_OFFSETS:
            yield tz
    else:
        for tz in _defs._ALL_TIMEZONES:
            yield tz


def get_timezones_dict():
    global _ALL_TIMEZONES_DICT
    if _ALL_TIMEZONES_DICT is None:
        _ALL_TIMEZONES_DICT = {tz[1]: tz for tz in get_timezones()}
    return _ALL_TIMEZONES_DICT



_ALL_TIMEZONES_DICT = None


def _tz_offset_key(offset):
    # Convert a tz offset to a key that can be used by sort().
    # Since Python can sort tuples in lexicographical order, we just convert it
    # to a tuple.
    if isinstance(offset, tuple):
        offset = offset[0]

    mtch = _RE_TZ_OFFSET.match(offset)
    is_negative, hours, minutes = (
        mtch.group(1) == "-",
        int(mtch.group(2)),
        int(mtch.group(3)),
    )

    if is_negative:
        hours, minutes = -hours, -minutes

    return (hours, minutes)


def _update_offsets(timezone_collection):
    new_collection = []

    for _, name, tz_formated in timezone_collection:
        new_collection.append(tz_utils.format_tz_by_name(name, tz_formated))

    return sorted(new_collection, key=_tz_offset_key)



import datetime
import os.path

import pytest

from timezones import _defs, tz_rendering, tz_utils, zones

GEOIP_DATA_LOCATION = os.path.abspath(
    os.path.join(__file__, "..", "..", "GeoIP2-City-Test.mmdb")
)


def assert_is_lower(offset_a, offset_b):
    # Force a real sort to happen: to avoid optimizations from kicking in, make
    # sure the list is really out of the expected order, and not in reverse
    # order either.
    coll = ["+9999", offset_b, "-9999", offset_a]
    coll.sort(key=zones._tz_offset_key)
    assert coll[1] == offset_a


def test_sort():
    assert_is_lower("+0100", "+0400")
    assert_is_lower("+0400", "+0430")
    assert_is_lower("+0430", "+0500")

    assert_is_lower("-1000", "+0430")
    assert_is_lower("-1030", "-0500")

    assert_is_lower("-1030", "+0000")

    assert_is_lower("-0030", "+0030")

    assert_is_lower("-0045", "-0030")
    assert_is_lower("+0030", "+0045")


def test_get_timezone():
    assert tz_utils.get_timezone("Europe/Moscow") is not None
    assert tz_utils.get_timezone("Europe/Moscow1") is None
    assert tz_utils.get_timezone("GMT +1:00") is not None

    assert tz_utils.is_valid_timezone("GMT +1:00")
    assert tz_utils.is_valid_timezone("Europe/Moscow")
    assert tz_utils.is_valid_timezone("Europe/Moscow1") is False


@pytest.fixture
def geoip_db():
    try:
        import geoip2
    except ImportError:
        pytest.skip("geoip2 not installed")
    if not os.path.exists(GEOIP_DATA_LOCATION):
        pytest.skip("No GeoIP2 database")

    tz_utils.GEOIP_DATA_LOCATION = GEOIP_DATA_LOCATION
    yield
    tz_utils.GEOIP_DATA_LOCATION = None


def test_guess_timezone_geoip(geoip_db):
    assert tz_utils.guess_timezone_by_ip("000.000.000.000", only_name=True) is None
    assert tz_utils.guess_timezone_by_ip("127.0.0.1", only_name=True) is None

    # This uses the MaxMind test DBs
    # Source: https://github.com/maxmind/MaxMind-DB/blob/main/source-data/GeoIP2-City-Test.json
    assert (
        tz_utils.guess_timezone_by_ip("149.101.100.0", only_name=True)
        == "America/Chicago"
    )
    assert tz_utils.guess_timezone_by_ip("2001:230::1", only_name=True) == "Asia/Seoul"

    # Addresses not in the test DB
    assert tz_utils.guess_timezone_by_ip("1.2.3.4", only_name=True) is None
    assert (
        tz_utils.guess_timezone_by_ip("2001:2002:2003:2004::0", only_name=True) is None
    )


def test_guess_timezone_no_geoip():
    assert tz_utils.guess_timezone_by_ip("149.101.100.0", only_name=True) is None
    assert tz_utils.guess_timezone_by_ip("2001:230::1", only_name=True) is None


def test_get_timezones():
    assert len(list(zones.get_timezones(only_us=True))) == 8


@pytest.mark.parametrize("offset_str,tzname,verbose_name", _defs._ALL_TIMEZONES)
def test_valid_offset(offset_str, tzname, verbose_name):
    assert tz_utils.is_valid_timezone(tzname)
    tz = tz_utils.get_timezone(tzname)

    # 1. Find a timestamp without DST shift
    now = datetime.datetime.utcnow()
    winter_time = datetime.datetime(now.year + 1, 1, 1)
    summer_time = datetime.datetime(now.year + 1, 7, 1)

    # Looking for a timestamp with zero DST offset
    for ts in [winter_time, summer_time]:
        if tz.dst(ts) == datetime.timedelta(0):
            break

    # 2. Take tz shift without DST
    offset_full_minutes = int(tz.utcoffset(ts).total_seconds() / 60)
    offset_sign = "+" if offset_full_minutes >= 0 else "-"

    offset_hours = abs(offset_full_minutes) // 60
    offset_minutes = abs(offset_full_minutes) - (offset_hours * 60)
    expected_offset = "%s%02d%02d" % (offset_sign, offset_hours, offset_minutes)
    assert offset_str == expected_offset, "Invalid offset for {}".format(tzname)

    # 3. Test verbose name
    assert verbose_name.startswith("(GMT%s) " % expected_offset)


def test_get_timezones_json():
    json_list = tz_rendering.get_timezones_json()
    assert "US/" in json_list


@pytest.mark.parametrize("tzname", _defs._PYTZ_ALIASES.keys())
def test_aliases(tzname):
    _, name, formatted = tz_utils.format_tz_by_name(tzname)
    assert name == tzname
    assert tzname in formatted



# -*- coding: utf-8 -*-
"""
tz_utils
~~~~~~~~

Includes timezone related utilities.


Example usage (get a fixed offset timezone)::

    print tz_utils.get_timezone('GMT +10:00')


Example usage (guess timezone by IP, required geoip2!)::

    tz_utils.GEOIP_DATA_LOCATION = '/usr/local/geo_ip/GeoIP2-City.mmdb'
    assert tz_utils.guess_timezone_by_ip('201.246.115.62') == 'Chile/Continental'


Example usage (format timezone by name)::

    print tz_utils.format_tz_by_name('Europe/Copenhagen')
        =>
    ("+0100", "Europe/Copenhagen", '(GMT+0100) Copenhagen')


Example usage (is a timezone valid?)::

    print tz_utils.is_valid_timezone('Europe/Copenhagen')
        =>
    True


:copyright: 2012 by Amir Salihefendic ( http://amix.dk/ )
:license: MIT
"""
import re
from datetime import datetime, timedelta, tzinfo

import pytz

from timezones import _defs

try:
    import geoip2.database as geoip2_db

    HAS_GEOIP2 = True
except ImportError:
    HAS_GEOIP2 = False

# --- Exports ----------------------------------------------
__all__ = [
    "get_timezone",
    "is_valid_timezone",
    "GEOIP_DATA_LOCATION",
    "guess_timezone_by_ip",
    "format_tz_by_name",
]

# --- Specifies the location of GeoIP GeoLiteCity.dat database ---
GEOIP_DATA_LOCATION = None


# --- Functions ----------------------------------------------
def guess_timezone_by_ip(ip, only_name=False):
    """Given an `ip` with guess timezone using geoip2.
    Returns a tuple of (tz_offets, tz_name, tz_formated).
    `None` is returned if it can't guess a timezone.

    For this to work you need to set tz_utils.GEOIP_DATA_LOCATION
    You can get this database from http://www.maxmind.com/app/geolitecity

    Example usage::

        from timezones import tz_utils
        tz_utils.GEOIP_DATA_LOCATION = '/usr/local/geo_ip/GeoLiteCity.dat'
        assert tz_utils.guess_timezone_by_ip('201.246.115.62') == 'Chile/Continental'

    """
    geo_lib = _get_geoip_lib()
    if geo_lib:
        try:
            record = geo_lib.city(ip)
            if record:
                location = record.location
                if location and location.time_zone:
                    if only_name:
                        return location.time_zone
                    else:
                        return format_tz_by_name(location.time_zone)
        except:
            record = None
    return None


def get_timezone(tzname):
    """Get a timezone instance by name or return `None`.

    This getter support fixed offest timezone like `get_timezone('GMT +10:00')`"""
    try:
        # First, try with the provided name
        return pytz.timezone(tzname)
    except pytz.UnknownTimeZoneError:
        pass

    # No result: try with an alias, if there's one
    if alias := (_defs._PYTZ_ALIASES.get(tzname)):
        try:
            return pytz.timezone(alias)
        except pytz.UnknownTimeZoneError:
            pass

    # Still no result: fallback to a static timezone, or return None
    return _tz_map().get(tzname)


def is_valid_timezone(timezone):
    """Return `True` if the `timezone` is valid. Otherwise `False` is returned."""
    try:
        tz = get_timezone(timezone)
        if tz:
            return True
        else:
            return False
    except:
        return False


def format_tz_by_name(tz_name, tz_formated=None):
    """Returns a tuple of (tz_offets, tz_name, tz_formated).

    Example::
        format_tz_by_name('Europe/Copenhagen')
            =>
        ("+0100", "Europe/Copenhagen", '(GMT+0100) Copenhagen')
    """
    now = datetime.now(get_timezone(tz_name))
    offset = now.strftime("%z")

    if tz_formated:
        tz_formated = re.sub(r"\(GMT.+?\)", "(GMT%s)" % offset, tz_formated)
    else:
        tz_formated = "(GMT%s) %s" % (offset, tz_name)

    return (offset, tz_name, tz_formated)


# --- Private ----------------------------------------------
GEO_IP = None


def _get_geoip_lib():
    global GEO_IP

    if not HAS_GEOIP2 or not GEOIP_DATA_LOCATION:
        return None

    try:
        GEO_IP = geoip2_db.Reader(GEOIP_DATA_LOCATION)
    except:
        return None

    return GEO_IP


_zero = timedelta(0)


class FixedOffset(tzinfo):
    """Fixed offset in minutes east from UTC."""

    def __init__(self, offset, name):
        self.offset = offset
        self.name = name

        self._offset = timedelta(minutes=offset)
        self.zone = name

    def __str__(self):
        return self.zone

    def utcoffset(self, dt):
        return self._offset

    def tzname(self, dt):
        return self.zone

    def dst(self, dt):
        return _zero

    def localize(self, dt, is_dst=False):
        """Convert naive time to local time"""
        if dt.tzinfo is not None:
            raise ValueError("Not naive datetime (tzinfo is already set)")
        return dt.replace(tzinfo=self)

    def __getinitargs__(self):
        return (self.offset, self.name)


TZ_MAP = None


def _tz_map():
    global TZ_MAP

    if TZ_MAP is None:
        timezones = [
            FixedOffset(-720, "GMT -12:00"),
            FixedOffset(-660, "GMT -11:00"),
            FixedOffset(-600, "GMT -10:00"),
            FixedOffset(-540, "GMT -9:00"),
            FixedOffset(-480, "GMT -8:00"),
            FixedOffset(-420, "GMT -7:00"),
            FixedOffset(-360, "GMT -6:00"),
            FixedOffset(-300, "GMT -5:00"),
            FixedOffset(-240, "GMT -4:00"),
            FixedOffset(-180, "GMT -3:00"),
            FixedOffset(-120, "GMT -2:00"),
            FixedOffset(-60, "GMT -1:00"),
            FixedOffset(0, "GMT"),
            FixedOffset(60, "GMT +1:00"),
            FixedOffset(120, "GMT +2:00"),
            FixedOffset(180, "GMT +3:00"),
            FixedOffset(240, "GMT +4:00"),
            FixedOffset(300, "GMT +5:00"),
            FixedOffset(360, "GMT +6:00"),
            FixedOffset(420, "GMT +7:00"),
            FixedOffset(480, "GMT +8:00"),
            FixedOffset(540, "GMT +9:00"),
            FixedOffset(600, "GMT +10:00"),
            FixedOffset(660, "GMT +11:00"),
            FixedOffset(720, "GMT +12:00"),
            FixedOffset(780, "GMT +13:00"),
        ]

        TZ_MAP = dict([(z.zone, z) for z in timezones])

    return TZ_MAP



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['timezones']

package_data = \
{'': ['*']}

install_requires = \
['pytz']

extras_require = \
{'geoip': ['geoip2>=4.5.0,<5.0.0']}

setup_kwargs = {
    'name': 'timezones',
    'version': '2.1.0',
    'description': "A Python library that provides better selection of common timezones, can output HTML and auto select the best timezone based on user's IP.",
    'long_description': "python-timezones\n----------------\n\n![](https://github.com/Doist/python-timezones/workflows/Tests/badge.svg)\n\nA Python library that provides better selection of common timezones,\ncan output HTML and auto select the best timezone based on user's IP.\n\nVisit https://doist.github.io/python-timezones/ for more information.\n\nCopyright: 2012-2022 by Doist\nLicense: MIT.\n",
    'author': 'Doist Developers',
    'author_email': 'dev@doist.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://doist.com',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
